#include <stdlib.h>
#include <osgDB/FileUtils>
#include <fstream>
#include "appconfig.h"
#include <string>

CfgGrid::CfgGrid() {
    unitSize=1.0f;
    edgeSize=50.0f;
    grid_color=osg::Vec4(0.75f,0.75f,0.75f,1.0f);
    axis_color=osg::Vec4(1.0f,1.0f,1.0f,1.0f);
}

CfgView::CfgView() {
    axis_system=CameraView::Y_UP;
    projection_type=CameraView::ORTHOGONAL;
    view_type=CameraView::FRONT;
}

AppConfig *AppConfig::_instance=new AppConfig();

AppConfig::AppConfig() {
    background_color=osg::Vec4(0.5, 0.5, 0.5, 1.0f);
    grid=new CfgGrid();
    view=new CfgView();
}

void AppConfig::read() {
    std::string file;    
    osgDB::FilePathList path;
    osgDB::FilePathList::iterator iter;
    
#ifndef _MSC_VER   
    std::string home = getenv("HOME");
    std::string etc = "/etc";
#else
    std::string home = "";
    std::string etc = "";
#endif

    path.push_back(home + "/.osgeditrc");
    path.push_back(etc + "/osgedit.cfg");
    path.push_back(etc + "/osgeditrc");

    for (iter = path.begin(); iter != path.end(); iter++) {
	if (osgDB::fileExists(*iter)) {
	    std::ifstream in(iter->c_str());
	    read(in);
            return;
	}
    }
}

void AppConfig::write() {
#ifndef _MSC_VER
    std::string home = getenv("HOME");
#else
    std::string home = "";
#endif
    std::string file = home + "/.osgeditrc";
    std::ofstream out(file.c_str());
    write(out);
    out.close();
}

void AppConfig::read(std::istream &in) {
    std::string name;
    in >> name >> background_color[0] >> background_color[1] >> background_color[2] >> background_color[3];
    grid->read(in);
    view->read(in);
}

void AppConfig::write(std::ostream &out) {
    out << "background_color " << background_color << std::endl;
    grid->write(out);
    view->write(out);
}

void CfgGrid::read(std::istream &in) {
    std::string name;
    in >> name >> unitSize;
    in >> name >> edgeSize;
    in >> name >> grid_color[0] >> grid_color[1] >> grid_color[2] >> grid_color[3];
    in >> name >> axis_color[0] >> axis_color[1] >> axis_color[2] >> axis_color[3];
}

void CfgGrid::write(std::ostream &out) {
    out << "unitSize " << unitSize << std::endl;
    out << "edgeSize " << edgeSize << std::endl;
    out << "grid_color " << grid_color << std::endl;
    out << "axis_color " << axis_color << std::endl;
}

void CfgView::read(std::istream &in) {
    std::string name;
    in >> name >> (int&)axis_system;in.get();
    in >> name >> (int&)projection_type;in.get();
    in >> name >> (int&)view_type;in.get();
}

void CfgView::write(std::ostream &out) {
    out << "axis_system " << axis_system << std::endl;
    out << "projection_type " << projection_type << std::endl;
    out << "view_type " << view_type << std::endl;
}
