#include "comstateset.h"
#include <iostream>
#include <functional>
#include <algorithm>

typedef std::pair<osg::StateAttribute::GLMode,osg::StateAttribute::GLModeValue> StatePair;
class isMode: public std::unary_function<StatePair,bool > {
public:
    isMode(osg::StateAttribute::GLMode mode) { this->mode = mode; }
    bool operator()(StatePair p) {
	return p.first == mode;
    }
private: osg::StateAttribute::GLMode mode;
};

/** Syncs the state attributes with the modes, adding new modes enabled
 * for the new state attributes added by the user */
void ComStateSet::addModes(osg::StateSet::ModeList &modes, AttributeList &attributes) {
    std::vector<osg::StateAttribute::GLMode> the_modes;
    for (unsigned i=0;i<attributes.size();i++) {
        attributes[i]->getAssociatedModes(the_modes);
    }
    for (unsigned i=0;i<the_modes.size();i++) {
	if (std::find_if(modes.begin(),modes.end(),isMode(the_modes[i]))==modes.end()) {
	    modes[the_modes[i]] = osg::StateAttribute::ON;
	}
    }
}

ComStateSet::ComStateSet(osg::StateSet *node,
			 osg::StateSet::ModeList modes,
			 AttributeList attributes,
			 osg::StateSet::TextureModeList texture_modes,
			 TextureAttributeList texture_attributes,
			 int rendering_hint) {
    this->affected=node;

    this->modes_old = node->getModeList();
    this->attributes_old = node->getAttributeList();
    this->texture_modes_old = node->getTextureModeList();
    this->texture_attributes_old = node->getTextureAttributeList();
    this->rendering_hint_old = (int)node->getRenderingHint();
    this->modes_new = modes;
    this->attributes_new = attributes;
    this->texture_modes_new = texture_modes;
    this->texture_attributes_new = texture_attributes;
    this->rendering_hint_new = rendering_hint;

    // Syncing the state attributes with the modes
    addModes(modes_new, attributes_new);
    for (unsigned i=0;i<texture_attributes_new.size();i++) {
	while (texture_modes_new.size() <= i)
	    texture_modes_new.push_back(osg::StateSet::ModeList());
        addModes(texture_modes_new[i], texture_attributes_new[i]);
    }
}

void ComStateSet::realExecute() {
    osg::StateSet::ModeList::iterator miter;
    for (miter=modes_new.begin();miter != modes_new.end();miter++) {
        affected->setMode(miter->first, miter->second);
    }
    for (unsigned i=0;i<attributes_new.size();i++) {
        affected->setAttribute(attributes_new[i].get());
    }
    for (unsigned unit = 0; unit < texture_modes_new.size(); unit++) {
	for (miter=texture_modes_new[unit].begin();miter != texture_modes_new[unit].end();miter++) {
            affected->setTextureMode(unit,miter->first, miter->second);
	}
    }
    for (unsigned unit=0;unit<texture_attributes_new.size();unit++) {
	for (unsigned i=0;i<texture_attributes_new[unit].size();i++) {
	    affected->setTextureAttribute(unit,texture_attributes_new[unit][i].get());
	}
    }

    affected->setRenderingHint(rendering_hint_new);
}

void ComStateSet::undo() {
    osg::StateSet::ModeList::iterator miter;
    for (miter=modes_old.begin();miter != modes_old.end();miter++) {
        affected->setMode(miter->first, miter->second);
    }
    osg::StateSet::AttributeList::iterator aiter;
    for (aiter=attributes_old.begin();aiter != attributes_old.end();aiter++) {
        affected->setAttribute(aiter->second.first.get());
    }
    for (unsigned unit = 0; unit < texture_modes_old.size(); unit++) {
	for (miter=texture_modes_old[unit].begin();miter != texture_modes_old[unit].end();miter++) {
            affected->setTextureMode(unit,miter->first, miter->second);
	}
    }
    for (unsigned unit = 0; unit < texture_attributes_old.size(); unit++) {
	for (aiter=texture_attributes_old[unit].begin();aiter != texture_attributes_old[unit].end();aiter++) {
            affected->setTextureAttribute(unit, aiter->second.first.get());
	}
    }

    affected->setRenderingHint(rendering_hint_old);
}

