#include <iostream>
#include <editable/editableentity.h>
#include <stdio.h>
#include "edreference.h"

EdReference::EdReference(std::string name,
			 std::string description):
    EditableAttribute(name, description)
{
    // By default don't allow changing, there aren't classes...
    allow_change=false;
}

void EdReference::set(std::string the_class) {
    if (map.find(the_class) != map.end()) {
	value=map[the_class]->clone();
        value->genObject();
	// FIXME: value=_value;
	notifyChange();
    } else {
	std::cerr << "FATAL ERROR: EdReference::set: Unknow class '" << the_class << "'" << std::endl;
    }
}

void EdReference::setInitial(EditableEntity *entity) {
    original=entity;
    value=entity;
}

void EdReference::setNull() {
    value=0;
}

std::string EdReference::toString() const {
    if (value.valid()) {
	return value->getName();
    } else {
        return "NULL";
    }
}

void EdReference::addClass(std::string name, EditableEntity *prototype) {
    map[name]=prototype;
}

std::vector<std::string> EdReference::getClasses() {
    std::map<std::string, osg::ref_ptr<EditableEntity> >::iterator iter;
    std::vector<std::string> result;
    for (iter=map.begin(); iter != map.end(); iter++) {
        result.push_back(iter->first);
    }
    return result;
}

EditableAttribute *EdReference::clone() const {
    EdReference *clon=new EdReference(getName(), getDescription());
    std::map<std::string, osg::ref_ptr<EditableEntity> >::const_iterator iter;
    for (iter=map.begin(); iter != map.end(); iter++) {
        clon->addClass(iter->first, iter->second->clone());
    }
    clon->setAllowChange(allow_change);
    /** If only one possible type and the user isn't allowed to change it,
     * instantiate now to avoid a permanent NULL reference */
    if (map.size() == 1 && !allow_change) {
        clon->setInitial(map.begin()->second->clone());
    }
    return clon;
}
