#include <editable/entityfactory.h>
#include "edstateset.h"
#include "edgeometry.h"

#include <command/comgeometry.h>


void EdGeometry::setObject(osg::Object *object) {
    model=dynamic_cast<osg::Geometry *> (object);
    if (model == 0) {
	std::cerr << "EdGeometry::setObject(): FATAL ERROR: This is not an 'osg::Geometry'" << std::endl;
        throw -1;
    }
    use_display_list = new EdBool( _("Use display list"), _("Use an OpenGL display list for this drawable"), model->getUseDisplayList() );
    list.push_back(use_display_list.get());

    use_vbo = new EdBool( _("Use VBO"), _("Use OpenGL vertex buffer objects"), model->getUseVertexBufferObjects() );
    list.push_back(use_vbo.get());

    stateset = new EdReference( _("State Set"), _("Geometry state set") );
    stateset->addClass(_("StateSet"), new EdStateSet);
    if (model->getStateSet()) {
	stateset->setInitial(EntityFactory::instance()->findEntity(model->getStateSet()));
    } else {
        stateset->setNull();
    }
    
    list.push_back(stateset.get());

}

EdGeometry::EdGeometry() {}

EdGeometry::EdGeometry(osg::Geometry *_model) {
    setObject(_model);
}

Command* EdGeometry::compile() {
	return new ComGeometry(model.get(),
            (bool)use_display_list->get(),
            (bool)use_vbo->get());
}
void EdGeometry::commit() {
    osg::ref_ptr<Command> com = compile();
    com->execute();
}
