#include <editable/entityfactory.h>
#include "edstateset.h"

#include "edshapedrawable.h"

#include <command/comshapedrawable.h>


void EdShapeDrawable::setObject(osg::Object *object) {
    model=dynamic_cast<osg::ShapeDrawable *> (object);
    if (model == 0) {
	std::cerr << "EdShapeDrawable::setObject(): FATAL ERROR: This is not an 'osg::ShapeDrawable'" << std::endl;
        throw -1;
    }
    use_display_list = new EdBool( _("Use display list"), _("Use an OpenGL display list for this drawable"), model->getUseDisplayList() );
    list.push_back(use_display_list.get());

    use_vbo = new EdBool( _("Use VBO"), _("Use OpenGL vertex buffer objects"), model->getUseVertexBufferObjects() );
    list.push_back(use_vbo.get());

    colour = new EdColour( _("Colour of the shape"), _("Colour for this shape") );
    colour->setInitial(model->getColor());
    list.push_back(colour.get());

    stateset = new EdReference( _("State Set"), _("ShapeDrawable state set") );
    stateset->addClass(_("StateSet"), new EdStateSet);
    if (model->getStateSet()) {
	stateset->setInitial(EntityFactory::instance()->findEntity(model->getStateSet()));
    } else {
        stateset->setNull();
    }
    
    list.push_back(stateset.get());

}

EdShapeDrawable::EdShapeDrawable() {}

EdShapeDrawable::EdShapeDrawable(osg::ShapeDrawable *_model) {
    setObject(_model);
}

Command* EdShapeDrawable::compile() {
	return new ComShapeDrawable(model.get(),
            (bool)use_display_list->get(),
            (bool)use_vbo->get(),
            (osg::Vec4)colour->get());
}

void EdShapeDrawable::commit() {
    osg::ref_ptr<Command> com = compile();
    com->execute();

}
