#include <config.h>
#include <iostream>
#include <gtk/gtk.h>
#include "gtkedlist.h"
#include "gtkedfactory.h"
#include <local.h>

#define VERTICAL_SPACING 0
#define HORIZONTAL_SPACING 3

void on_remove_pointer_clicked (GtkButton       *button,
				GtkEdList  *editor) {
    editor->removeItem();
}


void on_new_pointer_clicked (GtkButton       *button,
			     GtkEdList  *editor) {
    editor->addItem();
}

void GtkEdList::addItem() {
    unsigned line = attribute->addNew();
    GtkListStore *store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
    const EditableAttributeList &attributes = attribute->get(line);
    GtkTreeIter iter;
    gtk_list_store_append(store,&iter);
    gtk_list_store_set(store,&iter,0, line, -1);
    gtk_list_store_set(store,&iter,1, attribute->getItemName(line).c_str(), -1);
    for (unsigned i=2;i<attributes.size()+2; i++) {
		gtk_list_store_set(store,&iter,i,attributes[i-2]->toString().c_str(), -1);
    }
    select(line);
}

void GtkEdList::removeItem() {
    if (selected_row >=0)
	attribute->remove(selected_row);
    GtkListStore *store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
    GtkTreeIter iter;
    gtk_tree_model_get_iter_root(GTK_TREE_MODEL(store),&iter);
    for (int i=0;i<selected_row;++i) {
        gtk_tree_model_iter_next(GTK_TREE_MODEL(store),&iter);
    }
    
    gtk_list_store_remove(store, &iter);
    selected_row=-1;
}

void GtkEdList::setAttribute(EditableAttribute *attrib) {
    attribute=dynamic_cast<EdList*>(attrib);
    if (!attribute.valid()) {
	std::cerr << "GtkEdList::build: FATAL ERROR: argument isn't an EdList" << std::endl;
        throw -1;
    }

    /** Putting a label in the frame */
    gtk_frame_set_label (GTK_FRAME (widget), attribute->getName().c_str());
    // If has elements or prototype...
    if (attribute->length() != 0 || attribute->canAddRemove()) {
        const EditableAttributeList *attributes=&attribute->getPrototype();
	if (!attribute->canAddRemove()) {
            attributes=&attribute->get(0);
	}
	EditableAttributeList::const_iterator iter;
        /* Creating the editors for the different attributes in a line */
	for (iter=attributes->begin(); iter!=attributes->end(); iter++) {
	    osg::ref_ptr<EditableAttribute> attribute = *iter;
	    GtkEdAttribute *editor = GtkEdFactory::instance()->find(attribute.get());
	    attribute_editors.push_back(editor);
	    editor->build();
	    GtkWidget *editor_widget=editor->getWidget();

            if (editor->needsLabel()) {
		GtkWidget *hbox = gtk_hbox_new (FALSE, HORIZONTAL_SPACING);
		gtk_box_pack_start (GTK_BOX (editables), hbox, TRUE, TRUE, 0);
		GtkWidget *label = gtk_label_new (attribute->getName().c_str());
		gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, FALSE, 0);
		gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
		gtk_box_pack_start (GTK_BOX (hbox), editor_widget, FALSE, FALSE, 0);
	    } else {
		gtk_box_pack_start (GTK_BOX (editables), editor_widget, FALSE, FALSE, 0);
	    }
	}
        /* Defining data types of the GTK data model */
	GType *types=new GType[attributes->size()+2];
        types[0]=G_TYPE_INT;
	for (unsigned i=1;i<attributes->size()+2; i++) {
            types[i]=G_TYPE_STRING;
	}
        /* Iserting data into the GTK data model */
	GtkListStore *store = gtk_list_store_newv (attributes->size()+2, types);
	for (unsigned line=0; line <attribute->length(); line++) {
	    const EditableAttributeList &attributes = attribute->get(line);
	    GtkTreeIter iter;
	    gtk_list_store_append(store,&iter);
            gtk_list_store_set(store,&iter,0, line, -1);
            gtk_list_store_set(store,&iter,1, attribute->getItemName(line).c_str(), -1);
	    for (unsigned i=2;i<attributes.size()+2; i++) {
		gtk_list_store_set(store,&iter,i,attributes[i-2]->toString().c_str(), -1);
	    }
	}
	gtk_tree_view_set_model(GTK_TREE_VIEW(tree),GTK_TREE_MODEL(store));

	/* Defining tree view renderers */
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	for (unsigned i=0;i<=attributes->size();i++) {
	    renderer = gtk_cell_renderer_text_new();
	    std::string name;
	    if (i==0) name=_("Item");
            else name=(*attributes)[i-1]->getName();
	    column=gtk_tree_view_column_new_with_attributes (name.c_str(),
							     renderer,
							     "text", i+1,
							     NULL);
	    gtk_tree_view_append_column (GTK_TREE_VIEW(tree), column);
	}

    }
    if (attribute->canAddRemove()) { addButtons(); }
}

static void
tree_selection_changed_cb (GtkTreeSelection *selection, gpointer data) {
    GtkEdList *edlist=(GtkEdList*)data;

    GtkTreeIter iter;
    GtkTreeModel *model;
    if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
        int row;
	gtk_tree_model_get (model, &iter, 0, &row, -1);
        edlist->select(row);
    }

}

void GtkEdList::select(int row) {
    EditableAttributeList::iterator iter_obs;
    for (iter_obs=observed.begin();iter_obs!=observed.end();iter_obs++) {
	EditableAttribute *attr=iter_obs->get();
        attr->removeObserver(this);
    }
    while(observed.size()) observed.erase(observed.begin());
    /* Setting the new editable attributes for the editors */
    const EditableAttributeList &attributes = attribute->get(row);
    EditableAttributeList::const_iterator iter;
    EditorList::iterator iter_editor=attribute_editors.begin();
    for (iter=attributes.begin(); iter!=attributes.end(); iter++, iter_editor++) {
	osg::ref_ptr<EditableAttribute> attribute = *iter;
	osg::ref_ptr<GtkEdAttribute> editor = *iter_editor;
	editor->setAttribute(attribute.get());
	observed.push_back(attribute);
        attribute->addObserver(this);
    }
    this->selected_row=row;
}

void GtkEdList::build() {
    widget = gtk_frame_new (NULL);

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
    gtk_container_add (GTK_CONTAINER (widget), vbox);

    GtkWidget *scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
    gtk_box_pack_start (GTK_BOX (vbox), scrolledwindow1, TRUE, TRUE, 0);

    tree = gtk_tree_view_new ();
	gtk_widget_set_size_request(tree, -1, 100);
    gtk_container_add (GTK_CONTAINER (scrolledwindow1), tree);

    GtkTreeSelection *select;

    select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
    g_signal_connect (G_OBJECT (select), "changed",
		      G_CALLBACK (tree_selection_changed_cb),this);


    editables = gtk_vbox_new (FALSE, VERTICAL_SPACING);
    gtk_container_add (GTK_CONTAINER (vbox), editables);


    gtk_widget_ref(widget);
}

GtkEdList::GtkEdList() {
    widget=0;
    tree=0;
    editables=0;
    vbox=0;
    buttons=0;
    selected_row=-1;
}

GtkEdList::~GtkEdList() {
    if (widget!=0) gtk_widget_unref(widget);
}

void GtkEdList::attributeChanged(EditableAttribute *source) {
    GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
    GtkTreeIter iter;
    GtkTreeModel *model;
    if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
	for (unsigned i=0;i<observed.size();i++) {
	    if (observed[i] == source) {
		gtk_list_store_set (GTK_LIST_STORE(model), &iter, 2+i, source->toString().c_str(), -1);
	    }
	}
    }
}

void GtkEdList::addButtons() {
    GtkWidget *hbox1 = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox1, FALSE, TRUE, 0);

    GtkWidget *remove_pointer = gtk_button_new ();
    gtk_box_pack_end (GTK_BOX (hbox1), remove_pointer, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (remove_pointer), 2);

    GtkWidget *alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
    gtk_container_add (GTK_CONTAINER (remove_pointer), alignment2);

    GtkWidget *hbox3 = gtk_hbox_new (FALSE, 2);
    gtk_container_add (GTK_CONTAINER (alignment2), hbox3);

    GtkWidget *r_pointer = gtk_image_new_from_stock ("gtk-delete", GTK_ICON_SIZE_BUTTON);
    gtk_box_pack_start (GTK_BOX (hbox3), r_pointer, FALSE, FALSE, 0);

    GtkWidget *label4 = gtk_label_new_with_mnemonic (_("Remove"));
    gtk_box_pack_start (GTK_BOX (hbox3), label4, FALSE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (label4), GTK_JUSTIFY_LEFT);

    GtkWidget *new_pointer = gtk_button_new ();
    gtk_box_pack_end (GTK_BOX (hbox1), new_pointer, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (new_pointer), 2);

    GtkWidget *alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
    gtk_container_add (GTK_CONTAINER (new_pointer), alignment1);

    GtkWidget *hbox2 = gtk_hbox_new (FALSE, 2);
    gtk_container_add (GTK_CONTAINER (alignment1), hbox2);

    GtkWidget *image1 = gtk_image_new_from_stock ("gtk-new", GTK_ICON_SIZE_BUTTON);
    gtk_box_pack_start (GTK_BOX (hbox2), image1, FALSE, FALSE, 0);

    GtkWidget *label3 = gtk_label_new_with_mnemonic (_("Add"));
    gtk_box_pack_start (GTK_BOX (hbox2), label3, FALSE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_LEFT);

    g_signal_connect ((gpointer) remove_pointer, "clicked",
		      G_CALLBACK (on_remove_pointer_clicked),
		      this);
    g_signal_connect ((gpointer) new_pointer, "clicked",
		      G_CALLBACK (on_new_pointer_clicked),
		      this);
    buttons=hbox1;
}
