#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifndef VERSION
#define VERSION "0.5.0"
#endif

#include <sys/types.h>
#include <sys/stat.h>
#ifndef WIN32
#  include <unistd.h>
#endif
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "callbacks.h"
#include "interface.h"
#include "gui/support.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

void
on_about_dialog_destroy                (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_widget_destroy (GTK_WIDGET(user_data));
}

GtkWidget*
create_about_window (void)
{
  GtkWidget *about_dialog;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox2;
  GtkWidget *image52;
  GtkWidget *label1;
  GtkWidget *label2;
  GtkWidget *label3;
  GtkWidget *hseparator2;
  GtkWidget *label4;
  GtkWidget *label5;
  GtkWidget *hseparator3;
  GtkWidget *label6;
  GtkWidget *label7;
  GtkWidget *dialog_action_area1;
  GtkWidget *button1;

  about_dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (about_dialog), _("About OSGEdit"));

  dialog_vbox1 = GTK_DIALOG (about_dialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox2 = gtk_vbox_new (FALSE, 7);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox2, TRUE, TRUE, 0);

  image52 = create_pixmap (about_dialog, "osgedit/logo.png");
  gtk_widget_show (image52);
  gtk_box_pack_start (GTK_BOX (vbox2), image52, TRUE, TRUE, 0);

  label1 = gtk_label_new (_("Osgedit " VERSION));
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox2), label1, FALSE, FALSE, 0);

  label2 = gtk_label_new (_("OSGEdit is a scene compositor for \nOpen Scene Graph\n(http://www.openscenegraph.org)"));
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (vbox2), label2, FALSE, FALSE, 0);
  gtk_label_set_line_wrap (GTK_LABEL (label2), TRUE);

  label3 = gtk_label_new (_("(c) 2002 Ruben Lopez"));
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (vbox2), label3, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_LEFT);

  hseparator2 = gtk_hseparator_new ();
  gtk_widget_show (hseparator2);
  gtk_box_pack_start (GTK_BOX (vbox2), hseparator2, TRUE, TRUE, 0);

  label4 = gtk_label_new (_("Written by :"));
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (vbox2), label4, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label4), GTK_JUSTIFY_LEFT);

  label5 = gtk_label_new (_("<b>Ruben Lopez\nRicardo Rodriguez</b>\nGTK2-only port by <b>Clement Bourdarias</b>\nEspecial thanks to the OSG team, \nespecially to Robert Osfield for his patience."));
  gtk_label_set_use_markup(GTK_LABEL(label5), TRUE);
  gtk_widget_show (label5);
  gtk_box_pack_start (GTK_BOX (vbox2), label5, FALSE, FALSE, 0);
  gtk_label_set_line_wrap (GTK_LABEL (label5), TRUE);

  hseparator3 = gtk_hseparator_new ();
  gtk_widget_show (hseparator3);
  gtk_box_pack_start (GTK_BOX (vbox2), hseparator3, TRUE, TRUE, 0);

  label6 = gtk_label_new (_("Translators :"));
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (vbox2), label6, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label6), GTK_JUSTIFY_LEFT);

  label7 = gtk_label_new (_("Ruben Lopez (Galician,Spanish)"));
  gtk_widget_show (label7);
  gtk_box_pack_start (GTK_BOX (vbox2), label7, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label7), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (label7), TRUE);
  gtk_misc_set_padding (GTK_MISC (label7), 0, 6);

  dialog_action_area1 = GTK_DIALOG (about_dialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  button1 = gtk_button_new_from_stock ("gtk-close");
  gtk_widget_show (button1);
  gtk_dialog_add_action_widget (GTK_DIALOG (about_dialog), button1, GTK_RESPONSE_CLOSE);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (on_about_dialog_destroy),
                      GTK_OBJECT (about_dialog));

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (about_dialog, about_dialog, "about_dialog");
  GLADE_HOOKUP_OBJECT_NO_REF (about_dialog, dialog_vbox1, "dialog_vbox1");
  GLADE_HOOKUP_OBJECT (about_dialog, vbox2, "vbox2");
  GLADE_HOOKUP_OBJECT (about_dialog, image52, "image52");
  GLADE_HOOKUP_OBJECT (about_dialog, label1, "label1");
  GLADE_HOOKUP_OBJECT (about_dialog, label2, "label2");
  GLADE_HOOKUP_OBJECT (about_dialog, label3, "label3");
  GLADE_HOOKUP_OBJECT (about_dialog, hseparator2, "hseparator2");
  GLADE_HOOKUP_OBJECT (about_dialog, label4, "label4");
  GLADE_HOOKUP_OBJECT (about_dialog, label5, "label5");
  GLADE_HOOKUP_OBJECT (about_dialog, hseparator3, "hseparator3");
  GLADE_HOOKUP_OBJECT (about_dialog, label6, "label6");
  GLADE_HOOKUP_OBJECT (about_dialog, label7, "label7");
  GLADE_HOOKUP_OBJECT_NO_REF (about_dialog, dialog_action_area1, "dialog_action_area1");
  GLADE_HOOKUP_OBJECT (about_dialog, button1, "button1");

  return about_dialog;
}

GtkWidget*
create_main_window (void)
{
  GtkWidget *main_window;
  GdkPixbuf *main_window_icon_pixbuf;
   GtkWidget *toolbar;
  GtkWidget *new2;
  GtkWidget *open;
  GtkWidget *save;
  GtkWidget *undo;
  GtkWidget *redo;
  GtkWidget *del;
  GtkWidget *tmp_toolbar_icon;
  GtkWidget *addTransform;
  GtkWidget *select;
  GtkWidget *move;
  GtkWidget *rotate;
  GtkWidget *scale;
  GtkWidget *zoom_in;
  GtkWidget *zoom_out;
  GtkWidget *orbit;
  GtkWidget *pan;
  GtkWidget *center_selection;
  GtkWidget *toolbar3;
  GtkWidget *do_sphere;
  GtkWidget *do_box;
  GtkWidget *do_cone;
  GtkWidget *do_cylinder;
  GtkWidget *hpaned1;
  GtkWidget *views;
  GtkWidget *vpaned1;
  GtkWidget *scrolll1;
  GtkWidget *scenegraph;
  GtkWidget *properties_scroll;
  GtkWidget *properties_viewport;

  // pure Gtk menus 
  GtkWidget *menubar1;
  GtkWidget *menuitem1;
  GtkWidget *menuitem1_menu;
  GtkWidget *new1;
  GtkWidget *open1;
  GtkWidget *merge1;
  GtkWidget *image116;
  GtkWidget *save1;
  GtkWidget *save_as1;
  GtkWidget *separatormenuitem1;
  GtkWidget *quit1;
  GtkWidget *menuitem2;
  GtkWidget *menuitem2_menu;
  GtkWidget *select1;
  GtkWidget *cut1;
  GtkWidget *copy1;
  GtkWidget *paste1;
  GtkWidget *delete1;
  GtkWidget *undo1;
  GtkWidget *redo1;
  GtkWidget *separator1;
  GtkWidget *properties1;
  GtkWidget *separator2;
  GtkWidget *preferences1;
  GtkWidget *view1;
  GtkWidget *view1_menu;
  GtkWidget *zoom_in1;
  GtkWidget *zoom_out1;
  GtkWidget *polygon_mode1;
  GtkWidget *polygon_mode1_menu;
  GtkWidget *solid1;
  GtkWidget *wireframe1;
  GtkWidget *shading1;
  GtkWidget *shading1_menu;
  GtkWidget *flat1;
  GtkWidget *smooth1;
  GtkWidget *toolbar1;
  GtkWidget *toolbar1_menu;
  GtkWidget *icons1;
  GtkWidget *icons_and_text1;
  GtkWidget *views1;
  GtkWidget *image117;
  GtkWidget *views1_menu;
  GtkWidget *front1;
  GtkWidget *back1;
  GtkWidget *left1;
  GtkWidget *right1;
  GtkWidget *top1;
  GtkWidget *bottom1;
  GtkWidget *user1;
  GtkWidget *projection1;
  GtkWidget *projection1_menu;
  GtkWidget *orthogonal1;
  GtkWidget *perspective1;
  GtkWidget *orbit1;
  GtkWidget *pan1;
  GtkWidget *center_selection1;
  GtkWidget *menuitem3;
  GtkWidget *menuitem3_menu;
  GtkWidget *move1;
  GtkWidget *rotate1;
  GtkWidget *scale1;
  GtkWidget *add1;
  GtkWidget *add1_menu;
  GtkWidget *group_node1;
  GtkWidget *image118;
  GtkWidget *transform_node1;
  GtkWidget *image119;
  GtkWidget *switch_node1;
  GtkWidget *image120;
  GtkWidget *lod_node1;
  GtkWidget *image121;
  GtkWidget *light_node1;
  GtkWidget *image122;
  GtkWidget *billboard_node1;
  GtkWidget *image123;
  GtkWidget *sequence1;
  GtkWidget *image124;
  GtkWidget *projection2;
  GtkWidget *geode1;
  GtkWidget *image125;
  GtkWidget *geode1_menu;
  GtkWidget *sphere1;
  GtkWidget *box1;
  GtkWidget *cone1;
  GtkWidget *cylinder1;
  GtkWidget *state1;
  GtkWidget *state1_menu;
  GtkWidget *state_set1;
  GtkWidget *material1;
  GtkWidget *blend_function1;
  GtkWidget *cull_face1;
  GtkWidget *front_face1;
  GtkWidget *light_model1;
  GtkWidget *shade_model1;
  GtkWidget *fog1;
  GtkWidget *vertex_program1;
  GtkWidget *fragment_program1;
  GtkWidget *menuitem4;
  GtkWidget *menuitem4_menu;
  GtkWidget *about1;
  
  GtkAccelGroup *accel_group;

  GtkWidget *statusbar1;
  GtkWidget *bigbox;

  main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (main_window), _("OSG Scene Editor (GTK2)"));

  gtk_widget_set_size_request (main_window, 800, 600);
  gtk_window_set_default_size (GTK_WINDOW (main_window), 800, 600);
  main_window_icon_pixbuf = create_pixbuf ("osgedit/icon.png");
  if (main_window_icon_pixbuf)
    {
      gtk_window_set_icon (GTK_WINDOW (main_window), main_window_icon_pixbuf);
      gdk_pixbuf_unref (main_window_icon_pixbuf);
    }


  bigbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (bigbox);
  
  // Pure GTK menus
  accel_group = gtk_accel_group_new ();
  menubar1 = gtk_menu_bar_new ();
  gtk_widget_show (menubar1);
  gtk_box_pack_start (GTK_BOX (bigbox), menubar1, FALSE, FALSE, 0);

  menuitem1 = gtk_menu_item_new_with_mnemonic (_("_File"));
  gtk_widget_show (menuitem1);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem1);

  menuitem1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem1), menuitem1_menu);

  new1 = gtk_image_menu_item_new_from_stock ("gtk-new", accel_group);
  gtk_widget_show (new1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), new1);

  open1 = gtk_image_menu_item_new_from_stock ("gtk-open", accel_group);
  gtk_widget_show (open1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), open1);

  merge1 = gtk_image_menu_item_new_with_mnemonic (_("_Merge"));
  gtk_widget_show (merge1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), merge1);

  image116 = gtk_image_new_from_stock ("gtk-add", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image116);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (merge1), image116);

  save1 = gtk_image_menu_item_new_from_stock ("gtk-save", accel_group);
  gtk_widget_show (save1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), save1);

  save_as1 = gtk_image_menu_item_new_from_stock ("gtk-save-as", accel_group);
  gtk_widget_show (save_as1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), save_as1);

  separatormenuitem1 = gtk_menu_item_new ();
  gtk_widget_show (separatormenuitem1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), separatormenuitem1);
  gtk_widget_set_sensitive (separatormenuitem1, FALSE);

  quit1 = gtk_image_menu_item_new_from_stock ("gtk-quit", accel_group);
  gtk_widget_show (quit1);
  gtk_container_add (GTK_CONTAINER (menuitem1_menu), quit1);

  menuitem2 = gtk_menu_item_new_with_mnemonic (_("_Edit"));
  gtk_widget_show (menuitem2);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem2);

  menuitem2_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem2), menuitem2_menu);

  select1 = gtk_menu_item_new_with_mnemonic (_("Select"));
  gtk_widget_show (select1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), select1);
  gtk_widget_add_accelerator (select1, "activate", accel_group,
                              GDK_Escape, GdkModifierType(0),
                              GTK_ACCEL_VISIBLE);

  cut1 = gtk_image_menu_item_new_from_stock ("gtk-cut", accel_group);
  gtk_widget_show (cut1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), cut1);

  copy1 = gtk_image_menu_item_new_from_stock ("gtk-copy", accel_group);
  gtk_widget_show (copy1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), copy1);

  paste1 = gtk_image_menu_item_new_from_stock ("gtk-paste", accel_group);
  gtk_widget_show (paste1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), paste1);

  delete1 = gtk_image_menu_item_new_from_stock ("gtk-clear", accel_group);
  gtk_widget_show (delete1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), delete1);

  undo1 = gtk_image_menu_item_new_from_stock ("gtk-undo", accel_group);
  gtk_widget_show (undo1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), undo1);

  redo1 = gtk_image_menu_item_new_from_stock ("gtk-redo", accel_group);
  gtk_widget_show (redo1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), redo1);

  separator1 = gtk_menu_item_new ();
  gtk_widget_show (separator1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), separator1);
  gtk_widget_set_sensitive (separator1, FALSE);

  properties1 = gtk_image_menu_item_new_from_stock ("gtk-properties", accel_group);
  gtk_widget_show (properties1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), properties1);

  separator2 = gtk_menu_item_new ();
  gtk_widget_show (separator2);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), separator2);
  gtk_widget_set_sensitive (separator2, FALSE);

  preferences1 = gtk_image_menu_item_new_from_stock ("gtk-preferences", accel_group);
  gtk_widget_show (preferences1);
  gtk_container_add (GTK_CONTAINER (menuitem2_menu), preferences1);

  view1 = gtk_menu_item_new_with_mnemonic (_("View"));
  gtk_widget_show (view1);
  gtk_container_add (GTK_CONTAINER (menubar1), view1);

  view1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (view1), view1_menu);

  zoom_in1 = gtk_image_menu_item_new_from_stock ("gtk-zoom-in", accel_group);
  gtk_widget_show (zoom_in1);
  gtk_container_add (GTK_CONTAINER (view1_menu), zoom_in1);

  zoom_out1 = gtk_image_menu_item_new_from_stock ("gtk-zoom-out", accel_group);
  gtk_widget_show (zoom_out1);
  gtk_container_add (GTK_CONTAINER (view1_menu), zoom_out1);

  polygon_mode1 = gtk_menu_item_new_with_mnemonic (_("Polygon mode"));
  gtk_widget_show (polygon_mode1);
  gtk_container_add (GTK_CONTAINER (view1_menu), polygon_mode1);

  polygon_mode1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (polygon_mode1), polygon_mode1_menu);

  solid1 = gtk_menu_item_new_with_mnemonic (_("Solid"));
  gtk_widget_show (solid1);
  gtk_container_add (GTK_CONTAINER (polygon_mode1_menu), solid1);
  gtk_widget_add_accelerator (solid1, "activate", accel_group,
                              GDK_W, 
			      GdkModifierType(GDK_SHIFT_MASK | GDK_MOD1_MASK),
                              GTK_ACCEL_VISIBLE);

  wireframe1 = gtk_menu_item_new_with_mnemonic (_("Wireframe"));
  gtk_widget_show (wireframe1);
  gtk_container_add (GTK_CONTAINER (polygon_mode1_menu), wireframe1);
  gtk_widget_add_accelerator (wireframe1, "activate", accel_group,
                              GDK_W, GDK_MOD1_MASK,
                              GTK_ACCEL_VISIBLE);

  shading1 = gtk_menu_item_new_with_mnemonic (_("Shading"));
  gtk_widget_show (shading1);
  gtk_container_add (GTK_CONTAINER (view1_menu), shading1);

  shading1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (shading1), shading1_menu);

  flat1 = gtk_menu_item_new_with_mnemonic (_("Flat"));
  gtk_widget_show (flat1);
  gtk_container_add (GTK_CONTAINER (shading1_menu), flat1);

  smooth1 = gtk_menu_item_new_with_mnemonic (_("Smooth"));
  gtk_widget_show (smooth1);
  gtk_container_add (GTK_CONTAINER (shading1_menu), smooth1);

  toolbar1 = gtk_menu_item_new_with_mnemonic (_("Toolbar"));
  gtk_widget_show (toolbar1);
  gtk_container_add (GTK_CONTAINER (view1_menu), toolbar1);

  toolbar1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (toolbar1), toolbar1_menu);

  icons1 = gtk_menu_item_new_with_mnemonic (_("Icons"));
  gtk_widget_show (icons1);
  gtk_container_add (GTK_CONTAINER (toolbar1_menu), icons1);

  icons_and_text1 = gtk_menu_item_new_with_mnemonic (_("Icons and text"));
  gtk_widget_show (icons_and_text1);
  gtk_container_add (GTK_CONTAINER (toolbar1_menu), icons_and_text1);

  views1 = gtk_image_menu_item_new_with_mnemonic (_("Views"));
  gtk_widget_show (views1);
  gtk_container_add (GTK_CONTAINER (view1_menu), views1);

  image117 = gtk_image_new_from_stock ("gtk-zoom-fit", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image117);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (views1), image117);

  views1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (views1), views1_menu);

  front1 = gtk_menu_item_new_with_mnemonic (_("Front"));
  gtk_widget_show (front1);
  gtk_container_add (GTK_CONTAINER (views1_menu), front1);
  gtk_widget_add_accelerator (front1, "activate", accel_group,
                              GDK_1, GdkModifierType(0),
                              GTK_ACCEL_VISIBLE);

  back1 = gtk_menu_item_new_with_mnemonic (_("Back"));
  gtk_widget_show (back1);
  gtk_container_add (GTK_CONTAINER (views1_menu), back1);
  gtk_widget_add_accelerator (back1, "activate", accel_group,
                              GDK_1, GDK_SHIFT_MASK,
                              GTK_ACCEL_VISIBLE);

  left1 = gtk_menu_item_new_with_mnemonic (_("Left"));
  gtk_widget_show (left1);
  gtk_container_add (GTK_CONTAINER (views1_menu), left1);
  gtk_widget_add_accelerator (left1, "activate", accel_group,
                              GDK_3, GDK_SHIFT_MASK,
                              GTK_ACCEL_VISIBLE);

  right1 = gtk_menu_item_new_with_mnemonic (_("Right"));
  gtk_widget_show (right1);
  gtk_container_add (GTK_CONTAINER (views1_menu), right1);
  gtk_widget_add_accelerator (right1, "activate", accel_group,
                              GDK_3, GdkModifierType(0),
                              GTK_ACCEL_VISIBLE);

  top1 = gtk_menu_item_new_with_mnemonic (_("Top"));
  gtk_widget_show (top1);
  gtk_container_add (GTK_CONTAINER (views1_menu), top1);
  gtk_widget_add_accelerator (top1, "activate", accel_group,
                              GDK_7, GdkModifierType(0),
                              GTK_ACCEL_VISIBLE);

  bottom1 = gtk_menu_item_new_with_mnemonic (_("Bottom"));
  gtk_widget_show (bottom1);
  gtk_container_add (GTK_CONTAINER (views1_menu), bottom1);
  gtk_widget_add_accelerator (bottom1, "activate", accel_group,
                              GDK_7, GDK_SHIFT_MASK,
                              GTK_ACCEL_VISIBLE);

  user1 = gtk_menu_item_new_with_mnemonic (_("User"));
  gtk_widget_show (user1);
  gtk_container_add (GTK_CONTAINER (views1_menu), user1);
  gtk_widget_add_accelerator (user1, "activate", accel_group,
                              GDK_5, GdkModifierType(0),
                              GTK_ACCEL_VISIBLE);

  projection1 = gtk_menu_item_new_with_mnemonic (_("Projection"));
  gtk_widget_show (projection1);
  gtk_container_add (GTK_CONTAINER (view1_menu), projection1);

  projection1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (projection1), projection1_menu);

  orthogonal1 = gtk_menu_item_new_with_mnemonic (_("Orthogonal"));
  gtk_widget_show (orthogonal1);
  gtk_container_add (GTK_CONTAINER (projection1_menu), orthogonal1);

  perspective1 = gtk_menu_item_new_with_mnemonic (_("Perspective"));
  gtk_widget_show (perspective1);
  gtk_container_add (GTK_CONTAINER (projection1_menu), perspective1);

  orbit1 = gtk_menu_item_new_with_mnemonic (_("Orbit"));
  gtk_widget_show (orbit1);
  gtk_container_add (GTK_CONTAINER (view1_menu), orbit1);
  gtk_widget_add_accelerator (orbit1, "activate", accel_group,
                              GDK_O, GDK_MOD1_MASK,
                              GTK_ACCEL_VISIBLE);

  pan1 = gtk_menu_item_new_with_mnemonic (_("Pan"));
  gtk_widget_show (pan1);
  gtk_container_add (GTK_CONTAINER (view1_menu), pan1);
  gtk_widget_add_accelerator (pan1, "activate", accel_group,
                              GDK_P, GDK_MOD1_MASK,
                              GTK_ACCEL_VISIBLE);

  center_selection1 = gtk_menu_item_new_with_mnemonic (_("Center selection"));
  gtk_widget_show (center_selection1);
  gtk_container_add (GTK_CONTAINER (view1_menu), center_selection1);
  gtk_widget_add_accelerator (center_selection1, "activate", accel_group,
                              GDK_C, GDK_MOD1_MASK,
                              GTK_ACCEL_VISIBLE);

  menuitem3 = gtk_menu_item_new_with_mnemonic (_("Transform"));
  gtk_widget_show (menuitem3);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem3);

  menuitem3_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem3), menuitem3_menu);

  move1 = gtk_menu_item_new_with_mnemonic (_("Move"));
  gtk_widget_show (move1);
  gtk_container_add (GTK_CONTAINER (menuitem3_menu), move1);
  gtk_widget_add_accelerator (move1, "activate", accel_group,
                              GDK_M, GDK_MOD1_MASK,
                              GTK_ACCEL_VISIBLE);

  rotate1 = gtk_menu_item_new_with_mnemonic (_("Rotate"));
  gtk_widget_show (rotate1);
  gtk_container_add (GTK_CONTAINER (menuitem3_menu), rotate1);
  gtk_widget_add_accelerator (rotate1, "activate", accel_group,
                              GDK_R, GDK_MOD1_MASK,
                              GTK_ACCEL_VISIBLE);

  scale1 = gtk_menu_item_new_with_mnemonic (_("Scale"));
  gtk_widget_show (scale1);
  gtk_container_add (GTK_CONTAINER (menuitem3_menu), scale1);
  gtk_widget_add_accelerator (scale1, "activate", accel_group,
                              GDK_S, GDK_MOD1_MASK,
                              GTK_ACCEL_VISIBLE);

  add1 = gtk_menu_item_new_with_mnemonic (_("Add"));
  gtk_widget_show (add1);
  gtk_container_add (GTK_CONTAINER (menubar1), add1);

  add1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (add1), add1_menu);

  group_node1 = gtk_image_menu_item_new_with_mnemonic (_("Group node"));
  gtk_widget_show (group_node1);
  gtk_container_add (GTK_CONTAINER (add1_menu), group_node1);

  image118 = create_pixmap (main_window, "osgedit/group.png");
  gtk_widget_show (image118);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (group_node1), image118);

  transform_node1 = gtk_image_menu_item_new_with_mnemonic (_("Transform node"));
  gtk_widget_show (transform_node1);
  gtk_container_add (GTK_CONTAINER (add1_menu), transform_node1);

  image119 = create_pixmap (main_window, "osgedit/transform.png");
  gtk_widget_show (image119);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (transform_node1), image119);

  switch_node1 = gtk_image_menu_item_new_with_mnemonic (_("Switch node"));
  gtk_widget_show (switch_node1);
  gtk_container_add (GTK_CONTAINER (add1_menu), switch_node1);

  image120 = create_pixmap (main_window, "osgedit/switch.png");
  gtk_widget_show (image120);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (switch_node1), image120);

  lod_node1 = gtk_image_menu_item_new_with_mnemonic (_("LOD node"));
  gtk_widget_show (lod_node1);
  gtk_container_add (GTK_CONTAINER (add1_menu), lod_node1);

  image121 = create_pixmap (main_window, "osgedit/lod.png");
  gtk_widget_show (image121);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (lod_node1), image121);

  light_node1 = gtk_image_menu_item_new_with_mnemonic (_("Light node"));
  gtk_widget_show (light_node1);
  gtk_container_add (GTK_CONTAINER (add1_menu), light_node1);

  image122 = create_pixmap (main_window, "osgedit/lightsource.png");
  gtk_widget_show (image122);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (light_node1), image122);

  billboard_node1 = gtk_image_menu_item_new_with_mnemonic (_("Billboard node"));
  gtk_widget_show (billboard_node1);
  gtk_container_add (GTK_CONTAINER (add1_menu), billboard_node1);

  image123 = create_pixmap (main_window, "osgedit/billboard.png");
  gtk_widget_show (image123);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (billboard_node1), image123);

  sequence1 = gtk_image_menu_item_new_with_mnemonic (_("Sequence"));
  gtk_widget_show (sequence1);
  gtk_container_add (GTK_CONTAINER (add1_menu), sequence1);

  image124 = create_pixmap (main_window, "osgedit/sequence.png");
  gtk_widget_show (image124);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (sequence1), image124);

  projection2 = gtk_menu_item_new_with_mnemonic (_("Projection"));
  gtk_widget_show (projection2);
  gtk_container_add (GTK_CONTAINER (add1_menu), projection2);

  geode1 = gtk_image_menu_item_new_with_mnemonic (_("Geode"));
  gtk_widget_show (geode1);
  gtk_container_add (GTK_CONTAINER (add1_menu), geode1);

  image125 = create_pixmap (main_window, "osgedit/geode.png");
  gtk_widget_show (image125);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (geode1), image125);

  geode1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (geode1), geode1_menu);

  sphere1 = gtk_menu_item_new_with_mnemonic (_("Sphere"));
  gtk_widget_show (sphere1);
  gtk_container_add (GTK_CONTAINER (geode1_menu), sphere1);

  box1 = gtk_menu_item_new_with_mnemonic (_("Box"));
  gtk_widget_show (box1);
  gtk_container_add (GTK_CONTAINER (geode1_menu), box1);

  cone1 = gtk_menu_item_new_with_mnemonic (_("Cone"));
  gtk_widget_show (cone1);
  gtk_container_add (GTK_CONTAINER (geode1_menu), cone1);

  cylinder1 = gtk_menu_item_new_with_mnemonic (_("Cylinder"));
  gtk_widget_show (cylinder1);
  gtk_container_add (GTK_CONTAINER (geode1_menu), cylinder1);

  state1 = gtk_menu_item_new_with_mnemonic (_("State"));
  gtk_widget_show (state1);
  gtk_container_add (GTK_CONTAINER (menubar1), state1);

  state1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (state1), state1_menu);

  state_set1 = gtk_menu_item_new_with_mnemonic (_("State set"));
  gtk_widget_show (state_set1);
  gtk_container_add (GTK_CONTAINER (state1_menu), state_set1);

  material1 = gtk_menu_item_new_with_mnemonic (_("Material"));
  gtk_widget_show (material1);
  gtk_container_add (GTK_CONTAINER (state1_menu), material1);

  blend_function1 = gtk_menu_item_new_with_mnemonic (_("Blend function"));
  gtk_widget_show (blend_function1);
  gtk_container_add (GTK_CONTAINER (state1_menu), blend_function1);

  cull_face1 = gtk_menu_item_new_with_mnemonic (_("Cull face"));
  gtk_widget_show (cull_face1);
  gtk_container_add (GTK_CONTAINER (state1_menu), cull_face1);

  front_face1 = gtk_menu_item_new_with_mnemonic (_("Front face"));
  gtk_widget_show (front_face1);
  gtk_container_add (GTK_CONTAINER (state1_menu), front_face1);

  light_model1 = gtk_menu_item_new_with_mnemonic (_("Light model"));
  gtk_widget_show (light_model1);
  gtk_container_add (GTK_CONTAINER (state1_menu), light_model1);

  shade_model1 = gtk_menu_item_new_with_mnemonic (_("Shade model"));
  gtk_widget_show (shade_model1);
  gtk_container_add (GTK_CONTAINER (state1_menu), shade_model1);

  fog1 = gtk_menu_item_new_with_mnemonic (_("Fog"));
  gtk_widget_show (fog1);
  gtk_container_add (GTK_CONTAINER (state1_menu), fog1);

  vertex_program1 = gtk_menu_item_new_with_mnemonic (_("Vertex program"));
  gtk_widget_show (vertex_program1);
  gtk_container_add (GTK_CONTAINER (state1_menu), vertex_program1);

  fragment_program1 = gtk_menu_item_new_with_mnemonic (_("Fragment program"));
  gtk_widget_show (fragment_program1);
  gtk_container_add (GTK_CONTAINER (state1_menu), fragment_program1);

  menuitem4 = gtk_menu_item_new_with_mnemonic (_("_Help"));
  gtk_widget_show (menuitem4);
  gtk_container_add (GTK_CONTAINER (menubar1), menuitem4);

  menuitem4_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem4), menuitem4_menu);

  about1 = gtk_menu_item_new_with_mnemonic (_("_About"));
  gtk_widget_show (about1);
  gtk_container_add (GTK_CONTAINER (menuitem4_menu), about1);
  
  // End of pure GTK menus
  toolbar = gtk_toolbar_new ();
  gtk_widget_show (toolbar);

  gtk_container_set_border_width (GTK_CONTAINER (toolbar), 1);
  gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);

  new2 = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
                                "gtk-new",
                                "gtk-new",
                                NULL, NULL, NULL, -1);
  gtk_widget_show (new2);

  open = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
                                "gtk-open",
                                "gtk-open",
                                NULL, NULL, NULL, -1);
  gtk_widget_show (open);

  save = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
                                "gtk-save",
                                "gtk-save",
                                NULL, NULL, NULL, -1);
  gtk_widget_show (save);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));

  undo = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
                                "gtk-undo",
                                "gtk-undo",
                                NULL, NULL, NULL, -1);
  gtk_widget_show (undo);

  redo = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
                                "gtk-redo",
                                "gtk-redo",
                                NULL, NULL, NULL, -1);
  gtk_widget_show (redo);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));

  del = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
                                "gtk-delete",
                                "gtk-delete",
                                NULL, NULL, NULL, -1);
  gtk_widget_show (del);

  tmp_toolbar_icon = create_pixmap (main_window, "osgedit/transform.png");
  addTransform = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Transform"),
                                _("Add a transform parent to the selected node"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar)->children)->data))->label), TRUE);
  gtk_widget_show (addTransform);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));

  tmp_toolbar_icon = create_pixmap (main_window, "osgedit/select.png");
  select = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Select"),
                                _("Select nodes by picking in the screen"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar)->children)->data))->label), TRUE);
  gtk_widget_show (select);

  tmp_toolbar_icon = create_pixmap (main_window, "osgedit/move.png");
  move = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Move"),
                                _("Move the selected transform"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar)->children)->data))->label), TRUE);
  gtk_widget_show (move);

  tmp_toolbar_icon = create_pixmap (main_window, "osgedit/rotate.png");
  rotate = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Rotate"),
                                _("Rotate the selected transform"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar)->children)->data))->label), TRUE);
  gtk_widget_show (rotate);

  tmp_toolbar_icon = create_pixmap (main_window, "osgedit/scale.png");
  scale = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Scale"),
                                _("Scale the selected transform"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar)->children)->data))->label), TRUE);
  gtk_widget_show (scale);

  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));

  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-zoom-in", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar)));
  zoom_in = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Zoom in"),
                                _("Zoom in"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar)->children)->data))->label), TRUE);
  gtk_widget_show (zoom_in);

  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-zoom-out", gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar)));
  zoom_out = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Zoom out"),
                                _("Zoom out"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar)->children)->data))->label), TRUE);
  gtk_widget_show (zoom_out);

  tmp_toolbar_icon = create_pixmap (main_window, "osgedit/orbit.png");
  orbit = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Orbit"),
                                _("Rotate camera"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar)->children)->data))->label), TRUE);
  gtk_widget_show (orbit);

  tmp_toolbar_icon = create_pixmap (main_window, "osgedit/move.png");
  pan = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Pan"),
                                _("Move camera focus"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar)->children)->data))->label), TRUE);
  gtk_widget_show (pan);

  tmp_toolbar_icon = create_pixmap (main_window, "osgedit/center.png");
  center_selection = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Center"),
                                _("Center camera on the selection"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar)->children)->data))->label), TRUE);
  gtk_widget_show (center_selection);

  toolbar3 = gtk_toolbar_new ();
  gtk_widget_show (toolbar3);

  gtk_toolbar_set_style (GTK_TOOLBAR (toolbar3), GTK_TOOLBAR_ICONS);

  tmp_toolbar_icon = create_pixmap (main_window, "osgedit/sphere.png");
  do_sphere = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar3),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Sphere"),
                                _("Create sphere"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar3)->children)->data))->label), TRUE);
  gtk_widget_show (do_sphere);

  tmp_toolbar_icon = create_pixmap (main_window, "osgedit/box.png");
  do_box = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar3),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Box"),
                                _("Create box"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar3)->children)->data))->label), TRUE);
  gtk_widget_show (do_box);

  tmp_toolbar_icon = create_pixmap (main_window, "osgedit/cone.png");
  do_cone = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar3),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Cone"),
                                _("Create cone"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar3)->children)->data))->label), TRUE);
  gtk_widget_show (do_cone);

  tmp_toolbar_icon = create_pixmap (main_window, "osgedit/cylinder.png");
  do_cylinder = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar3),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Cylinder"),
                                _("Create cylinder"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (toolbar3)->children)->data))->label), TRUE);
  gtk_widget_show (do_cylinder);

  gtk_box_pack_start (GTK_BOX (bigbox), toolbar, FALSE, FALSE,
		      0);
		      
  gtk_box_pack_start (GTK_BOX (bigbox), toolbar3, FALSE, FALSE,
		      0);

  hpaned1 = gtk_hpaned_new ();
  gtk_widget_show (hpaned1);

  gtk_box_pack_start (GTK_BOX (bigbox), hpaned1, TRUE, TRUE,
		      0);
		      
  gtk_container_add (GTK_CONTAINER (main_window), bigbox);
  
  gtk_paned_set_position (GTK_PANED (hpaned1), 450);

  views = gtk_frame_new (NULL);
  gtk_widget_show (views);
  gtk_paned_pack1 (GTK_PANED (hpaned1), views, FALSE, TRUE);
  gtk_frame_set_shadow_type (GTK_FRAME (views), GTK_SHADOW_NONE);

  vpaned1 = gtk_vpaned_new ();
  gtk_widget_show (vpaned1);
  gtk_paned_pack2 (GTK_PANED (hpaned1), vpaned1, TRUE, TRUE);

  scrolll1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolll1);
  gtk_paned_pack1 (GTK_PANED (vpaned1), scrolll1, FALSE, TRUE);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolll1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  scenegraph = gtk_tree_view_new ();
  gtk_widget_show (scenegraph);
  gtk_container_add (GTK_CONTAINER (scrolll1), scenegraph);

  properties_scroll = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (properties_scroll);
  gtk_paned_pack2 (GTK_PANED (vpaned1), properties_scroll, TRUE, TRUE);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (properties_scroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  properties_viewport = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (properties_viewport);
  gtk_container_add (GTK_CONTAINER (properties_scroll), properties_viewport);

  statusbar1 = gtk_statusbar_new ();
  gtk_widget_show (statusbar1);
  gtk_box_pack_start (GTK_BOX (bigbox), statusbar1, FALSE, FALSE, 0);
  g_signal_connect ((gpointer) main_window, "delete_event",
                    G_CALLBACK (on_main_window_delete_event),
                    NULL);

  g_signal_connect ((gpointer) new2, "clicked",
                    G_CALLBACK (on_new_clicked),
                    NULL);
  g_signal_connect ((gpointer) open, "clicked",
                    G_CALLBACK (on_open_clicked),
                    NULL);
  g_signal_connect ((gpointer) save, "clicked",
                    G_CALLBACK (on_save_clicked),
                    NULL);
  g_signal_connect ((gpointer) undo, "clicked",
                    G_CALLBACK (on_undo_clicked),
                    NULL);
  g_signal_connect ((gpointer) redo, "clicked",
                    G_CALLBACK (on_redo_clicked),
                    NULL);
  g_signal_connect ((gpointer) del, "clicked",
                    G_CALLBACK (on_delete_clicked),
                    NULL);
  g_signal_connect ((gpointer) addTransform, "clicked",
                    G_CALLBACK (on_addTransform_clicked),
                    NULL);
  g_signal_connect ((gpointer) select, "clicked",
                    G_CALLBACK (on_select_clicked),
                    NULL);
  g_signal_connect ((gpointer) move, "clicked",
                    G_CALLBACK (on_move_clicked),
                    NULL);
  g_signal_connect ((gpointer) rotate, "clicked",
                    G_CALLBACK (on_rotate_clicked),
                    NULL);
  g_signal_connect ((gpointer) scale, "clicked",
                    G_CALLBACK (on_scale_clicked),
                    NULL);
  g_signal_connect ((gpointer) zoom_in, "clicked",
                    G_CALLBACK (on_zoom_in_clicked),
                    NULL);
  g_signal_connect ((gpointer) zoom_out, "clicked",
                    G_CALLBACK (on_zoom_out_clicked),
                    NULL);
  g_signal_connect ((gpointer) orbit, "clicked",
                    G_CALLBACK (on_orbit_clicked),
                    NULL);
  g_signal_connect ((gpointer) pan, "clicked",
                    G_CALLBACK (on_pan_clicked),
                    NULL);
  g_signal_connect ((gpointer) center_selection, "clicked",
                    G_CALLBACK (on_center_selection_clicked),
                    NULL);
  g_signal_connect ((gpointer) do_sphere, "clicked",
                    G_CALLBACK (on_do_sphere_clicked),
                    NULL);
  g_signal_connect ((gpointer) do_box, "clicked",
                    G_CALLBACK (on_do_box_clicked),
                    NULL);
  g_signal_connect ((gpointer) do_cone, "clicked",
                    G_CALLBACK (on_do_cone_clicked),
                    NULL);
  g_signal_connect ((gpointer) do_cylinder, "clicked",
                    G_CALLBACK (on_do_cylinder_clicked),
                    NULL);
  g_signal_connect ((gpointer) scrolll1, "key_press_event",
                    G_CALLBACK (on_scenegraph_key_press_event),
                    NULL);

  g_signal_connect ((gpointer) new1, "activate",
                    G_CALLBACK (on_new1_activate),
                    NULL);
  g_signal_connect ((gpointer) open1, "activate",
                    G_CALLBACK (on_open1_activate),
                    NULL);
  g_signal_connect ((gpointer) merge1, "activate",
                    G_CALLBACK (on_merge1_activate),
                    NULL);
  g_signal_connect ((gpointer) save1, "activate",
                    G_CALLBACK (on_save1_activate),
                    NULL);
  g_signal_connect ((gpointer) save_as1, "activate",
                    G_CALLBACK (on_save_as1_activate),
                    NULL);
  g_signal_connect ((gpointer) quit1, "activate",
                    G_CALLBACK (on_quit1_activate),
                    NULL);
  g_signal_connect ((gpointer) select1, "activate",
                    G_CALLBACK (on_select1_activate),
                    NULL);
  g_signal_connect ((gpointer) cut1, "activate",
                    G_CALLBACK (on_cut1_activate),
                    NULL);
  g_signal_connect ((gpointer) copy1, "activate",
                    G_CALLBACK (on_copy1_activate),
                    NULL);
  g_signal_connect ((gpointer) paste1, "activate",
                    G_CALLBACK (on_paste1_activate),
                    NULL);
  g_signal_connect ((gpointer) delete1, "activate",
                    G_CALLBACK (on_clear1_activate),
                    NULL);
  g_signal_connect ((gpointer) undo1, "activate",
                    G_CALLBACK (on_undo_activate),
                    NULL);
  g_signal_connect ((gpointer) redo1, "activate",
                    G_CALLBACK (on_redo_activate),
                    NULL);
  g_signal_connect ((gpointer) properties1, "activate",
                    G_CALLBACK (on_properties1_activate),
                    NULL);
  g_signal_connect ((gpointer) preferences1, "activate",
                    G_CALLBACK (on_preferences1_activate),
                    NULL);
  g_signal_connect ((gpointer) zoom_in1, "activate",
                    G_CALLBACK (on_zoom_in1_activate),
                    NULL);
  g_signal_connect ((gpointer) zoom_out1, "activate",
                    G_CALLBACK (on_zoom_out1_activate),
                    NULL);
  g_signal_connect ((gpointer) solid1, "activate",
                    G_CALLBACK (on_solid1_activate),
                    NULL);
  g_signal_connect ((gpointer) wireframe1, "activate",
                    G_CALLBACK (on_wireframe1_activate),
                    NULL);
  g_signal_connect ((gpointer) flat1, "activate",
                    G_CALLBACK (on_flat1_activate),
                    NULL);
  g_signal_connect ((gpointer) smooth1, "activate",
                    G_CALLBACK (on_smooth1_activate),
                    NULL);
  g_signal_connect ((gpointer) icons1, "activate",
                    G_CALLBACK (on_icons1_activate),
                    NULL);
  g_signal_connect ((gpointer) icons_and_text1, "activate",
                    G_CALLBACK (on_icons_and_text1_activate),
                    NULL);
  g_signal_connect ((gpointer) front1, "activate",
                    G_CALLBACK (on_front1_activate),
                    NULL);
  g_signal_connect ((gpointer) back1, "activate",
                    G_CALLBACK (on_back1_activate),
                    NULL);
  g_signal_connect ((gpointer) left1, "activate",
                    G_CALLBACK (on_left1_activate),
                    NULL);
  g_signal_connect ((gpointer) right1, "activate",
                    G_CALLBACK (on_right1_activate),
                    NULL);
  g_signal_connect ((gpointer) top1, "activate",
                    G_CALLBACK (on_top1_activate),
                    NULL);
  g_signal_connect ((gpointer) bottom1, "activate",
                    G_CALLBACK (on_bottom1_activate),
                    NULL);
  g_signal_connect ((gpointer) user1, "activate",
                    G_CALLBACK (on_perspective1_activate),
                    NULL);
  g_signal_connect ((gpointer) orthogonal1, "activate",
                    G_CALLBACK (on_orthogonal1_activate),
                    NULL);
  g_signal_connect ((gpointer) perspective1, "activate",
                    G_CALLBACK (on_perspective2_activate),
                    NULL);
  g_signal_connect ((gpointer) orbit1, "activate",
                    G_CALLBACK (on_orbit1_activate),
                    NULL);
  g_signal_connect ((gpointer) pan1, "activate",
                    G_CALLBACK (on_pan1_activate),
                    NULL);
  g_signal_connect ((gpointer) center_selection1, "activate",
                    G_CALLBACK (on_center_selection1_activate),
                    NULL);
  g_signal_connect ((gpointer) move1, "activate",
                    G_CALLBACK (on_move1_activate),
                    NULL);
  g_signal_connect ((gpointer) rotate1, "activate",
                    G_CALLBACK (on_rotate1_activate),
                    NULL);
  g_signal_connect ((gpointer) scale1, "activate",
                    G_CALLBACK (on_scale1_activate),
                    NULL);
  g_signal_connect ((gpointer) group_node1, "activate",
                    G_CALLBACK (on_group1_activate),
                    NULL);
  g_signal_connect ((gpointer) transform_node1, "activate",
                    G_CALLBACK (on_transform1_activate),
                    NULL);
  g_signal_connect ((gpointer) switch_node1, "activate",
                    G_CALLBACK (on_switch1_activate),
                    NULL);
  g_signal_connect ((gpointer) lod_node1, "activate",
                    G_CALLBACK (on_lod1_activate),
                    NULL);
  g_signal_connect ((gpointer) light_node1, "activate",
                    G_CALLBACK (on_light_node1_activate),
                    NULL);
  g_signal_connect ((gpointer) billboard_node1, "activate",
                    G_CALLBACK (on_billboard_node1_activate),
                    NULL);
  g_signal_connect ((gpointer) sequence1, "activate",
                    G_CALLBACK (on_sequence1_activate),
                    NULL);
  g_signal_connect ((gpointer) projection2, "activate",
                    G_CALLBACK (on_projection2_activate),
                    NULL);
  g_signal_connect ((gpointer) sphere1, "activate",
                    G_CALLBACK (on_sphere1_activate),
                    NULL);
  g_signal_connect ((gpointer) box1, "activate",
                    G_CALLBACK (on_box1_activate),
                    NULL);
  g_signal_connect ((gpointer) cone1, "activate",
                    G_CALLBACK (on_cone1_activate),
                    NULL);
  g_signal_connect ((gpointer) cylinder1, "activate",
                    G_CALLBACK (on_cylinder1_activate),
                    NULL);
  g_signal_connect ((gpointer) state_set1, "activate",
                    G_CALLBACK (on_state_set1_activate),
                    NULL);
  g_signal_connect ((gpointer) material1, "activate",
                    G_CALLBACK (on_material1_activate),
                    NULL);
  g_signal_connect ((gpointer) blend_function1, "activate",
                    G_CALLBACK (on_blend_function1_activate),
                    NULL);
  g_signal_connect ((gpointer) cull_face1, "activate",
                    G_CALLBACK (on_cull_face1_activate),
                    NULL);
  g_signal_connect ((gpointer) front_face1, "activate",
                    G_CALLBACK (on_front_face1_activate),
                    NULL);
  g_signal_connect ((gpointer) light_model1, "activate",
                    G_CALLBACK (on_light_model1_activate),
                    NULL);
  g_signal_connect ((gpointer) shade_model1, "activate",
                    G_CALLBACK (on_shade_model1_activate),
                    NULL);
  g_signal_connect ((gpointer) fog1, "activate",
                    G_CALLBACK (on_fog1_activate),
                    NULL);
  g_signal_connect ((gpointer) vertex_program1, "activate",
                    G_CALLBACK (on_vertex_program1_activate),
                    NULL);
  g_signal_connect ((gpointer) fragment_program1, "activate",
                    G_CALLBACK (on_fragment_program1_activate),
                    NULL);
  g_signal_connect ((gpointer) about1, "activate",
                    G_CALLBACK (on_about1_activate),
                    NULL);
  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (main_window, main_window, "main_window");
  GLADE_HOOKUP_OBJECT (main_window, menubar1, "menubar1");
  GLADE_HOOKUP_OBJECT (main_window, menuitem1, "menuitem1");
  GLADE_HOOKUP_OBJECT (main_window, menuitem1_menu, "menuitem1_menu");
  GLADE_HOOKUP_OBJECT (main_window, new1, "new1");
  GLADE_HOOKUP_OBJECT (main_window, open1, "open1");
  GLADE_HOOKUP_OBJECT (main_window, merge1, "merge1");
  GLADE_HOOKUP_OBJECT (main_window, image116, "image116");
  GLADE_HOOKUP_OBJECT (main_window, save1, "save1");
  GLADE_HOOKUP_OBJECT (main_window, save_as1, "save_as1");
  GLADE_HOOKUP_OBJECT (main_window, separatormenuitem1, "separatormenuitem1");
  GLADE_HOOKUP_OBJECT (main_window, quit1, "quit1");
  GLADE_HOOKUP_OBJECT (main_window, menuitem2, "menuitem2");
  GLADE_HOOKUP_OBJECT (main_window, menuitem2_menu, "menuitem2_menu");
  GLADE_HOOKUP_OBJECT (main_window, select1, "select1");
  GLADE_HOOKUP_OBJECT (main_window, cut1, "cut1");
  GLADE_HOOKUP_OBJECT (main_window, copy1, "copy1");
  GLADE_HOOKUP_OBJECT (main_window, paste1, "paste1");
  GLADE_HOOKUP_OBJECT (main_window, delete1, "delete1");
  GLADE_HOOKUP_OBJECT (main_window, undo1, "undo1");
  GLADE_HOOKUP_OBJECT (main_window, redo1, "redo1");
  GLADE_HOOKUP_OBJECT (main_window, separator1, "separator1");
  GLADE_HOOKUP_OBJECT (main_window, properties1, "properties1");
  GLADE_HOOKUP_OBJECT (main_window, separator2, "separator2");
  GLADE_HOOKUP_OBJECT (main_window, preferences1, "preferences1");
  GLADE_HOOKUP_OBJECT (main_window, view1, "view1");
  GLADE_HOOKUP_OBJECT (main_window, view1_menu, "view1_menu");
  GLADE_HOOKUP_OBJECT (main_window, zoom_in1, "zoom_in1");
  GLADE_HOOKUP_OBJECT (main_window, zoom_out1, "zoom_out1");
  GLADE_HOOKUP_OBJECT (main_window, polygon_mode1, "polygon_mode1");
  GLADE_HOOKUP_OBJECT (main_window, polygon_mode1_menu, "polygon_mode1_menu");
  GLADE_HOOKUP_OBJECT (main_window, solid1, "solid1");
  GLADE_HOOKUP_OBJECT (main_window, wireframe1, "wireframe1");
  GLADE_HOOKUP_OBJECT (main_window, shading1, "shading1");
  GLADE_HOOKUP_OBJECT (main_window, shading1_menu, "shading1_menu");
  GLADE_HOOKUP_OBJECT (main_window, flat1, "flat1");
  GLADE_HOOKUP_OBJECT (main_window, smooth1, "smooth1");
  GLADE_HOOKUP_OBJECT (main_window, toolbar1, "toolbar1");
  GLADE_HOOKUP_OBJECT (main_window, toolbar1_menu, "toolbar1_menu");
  GLADE_HOOKUP_OBJECT (main_window, icons1, "icons1");
  GLADE_HOOKUP_OBJECT (main_window, icons_and_text1, "icons_and_text1");
  GLADE_HOOKUP_OBJECT (main_window, views1, "views1");
  GLADE_HOOKUP_OBJECT (main_window, image117, "image117");
  GLADE_HOOKUP_OBJECT (main_window, views1_menu, "views1_menu");
  GLADE_HOOKUP_OBJECT (main_window, front1, "front1");
  GLADE_HOOKUP_OBJECT (main_window, back1, "back1");
  GLADE_HOOKUP_OBJECT (main_window, left1, "left1");
  GLADE_HOOKUP_OBJECT (main_window, right1, "right1");
  GLADE_HOOKUP_OBJECT (main_window, top1, "top1");
  GLADE_HOOKUP_OBJECT (main_window, bottom1, "bottom1");
  GLADE_HOOKUP_OBJECT (main_window, user1, "user1");
  GLADE_HOOKUP_OBJECT (main_window, projection1, "projection1");
  GLADE_HOOKUP_OBJECT (main_window, projection1_menu, "projection1_menu");
  GLADE_HOOKUP_OBJECT (main_window, orthogonal1, "orthogonal1");
  GLADE_HOOKUP_OBJECT (main_window, perspective1, "perspective1");
  GLADE_HOOKUP_OBJECT (main_window, orbit1, "orbit1");
  GLADE_HOOKUP_OBJECT (main_window, pan1, "pan1");
  GLADE_HOOKUP_OBJECT (main_window, center_selection1, "center_selection1");
  GLADE_HOOKUP_OBJECT (main_window, menuitem3, "menuitem3");
  GLADE_HOOKUP_OBJECT (main_window, menuitem3_menu, "menuitem3_menu");
  GLADE_HOOKUP_OBJECT (main_window, move1, "move1");
  GLADE_HOOKUP_OBJECT (main_window, rotate1, "rotate1");
  GLADE_HOOKUP_OBJECT (main_window, scale1, "scale1");
  GLADE_HOOKUP_OBJECT (main_window, add1, "add1");
  GLADE_HOOKUP_OBJECT (main_window, add1_menu, "add1_menu");
  GLADE_HOOKUP_OBJECT (main_window, group_node1, "group_node1");
  GLADE_HOOKUP_OBJECT (main_window, image118, "image118");
  GLADE_HOOKUP_OBJECT (main_window, transform_node1, "transform_node1");
  GLADE_HOOKUP_OBJECT (main_window, image119, "image119");
  GLADE_HOOKUP_OBJECT (main_window, switch_node1, "switch_node1");
  GLADE_HOOKUP_OBJECT (main_window, image120, "image120");
  GLADE_HOOKUP_OBJECT (main_window, lod_node1, "lod_node1");
  GLADE_HOOKUP_OBJECT (main_window, image121, "image121");
  GLADE_HOOKUP_OBJECT (main_window, light_node1, "light_node1");
  GLADE_HOOKUP_OBJECT (main_window, image122, "image122");
  GLADE_HOOKUP_OBJECT (main_window, billboard_node1, "billboard_node1");
  GLADE_HOOKUP_OBJECT (main_window, image123, "image123");
  GLADE_HOOKUP_OBJECT (main_window, sequence1, "sequence1");
  GLADE_HOOKUP_OBJECT (main_window, image124, "image124");
  GLADE_HOOKUP_OBJECT (main_window, projection2, "projection2");
  GLADE_HOOKUP_OBJECT (main_window, geode1, "geode1");
  GLADE_HOOKUP_OBJECT (main_window, image125, "image125");
  GLADE_HOOKUP_OBJECT (main_window, geode1_menu, "geode1_menu");
  GLADE_HOOKUP_OBJECT (main_window, sphere1, "sphere1");
  GLADE_HOOKUP_OBJECT (main_window, box1, "box1");
  GLADE_HOOKUP_OBJECT (main_window, cone1, "cone1");
  GLADE_HOOKUP_OBJECT (main_window, cylinder1, "cylinder1");
  GLADE_HOOKUP_OBJECT (main_window, state1, "state1");
  GLADE_HOOKUP_OBJECT (main_window, state1_menu, "state1_menu");
  GLADE_HOOKUP_OBJECT (main_window, state_set1, "state_set1");
  GLADE_HOOKUP_OBJECT (main_window, material1, "material1");
  GLADE_HOOKUP_OBJECT (main_window, blend_function1, "blend_function1");
  GLADE_HOOKUP_OBJECT (main_window, cull_face1, "cull_face1");
  GLADE_HOOKUP_OBJECT (main_window, front_face1, "front_face1");
  GLADE_HOOKUP_OBJECT (main_window, light_model1, "light_model1");
  GLADE_HOOKUP_OBJECT (main_window, shade_model1, "shade_model1");
  GLADE_HOOKUP_OBJECT (main_window, fog1, "fog1");
  GLADE_HOOKUP_OBJECT (main_window, vertex_program1, "vertex_program1");
  GLADE_HOOKUP_OBJECT (main_window, fragment_program1, "fragment_program1");
  GLADE_HOOKUP_OBJECT (main_window, menuitem4, "menuitem4");
  GLADE_HOOKUP_OBJECT (main_window, menuitem4_menu, "menuitem4_menu");
  GLADE_HOOKUP_OBJECT (main_window, about1, "about1");
  GLADE_HOOKUP_OBJECT (main_window, toolbar, "toolbar");
  GLADE_HOOKUP_OBJECT (main_window, new2, "new2");
  GLADE_HOOKUP_OBJECT (main_window, open, "open");
  GLADE_HOOKUP_OBJECT (main_window, save, "save");
  GLADE_HOOKUP_OBJECT (main_window, undo, "undo");
  GLADE_HOOKUP_OBJECT (main_window, redo, "redo");
  GLADE_HOOKUP_OBJECT (main_window, del, "del");
  GLADE_HOOKUP_OBJECT (main_window, addTransform, "addTransform");
  GLADE_HOOKUP_OBJECT (main_window, select, "select");
  GLADE_HOOKUP_OBJECT (main_window, move, "move");
  GLADE_HOOKUP_OBJECT (main_window, rotate, "rotate");
  GLADE_HOOKUP_OBJECT (main_window, scale, "scale");
  GLADE_HOOKUP_OBJECT (main_window, zoom_in, "zoom_in");
  GLADE_HOOKUP_OBJECT (main_window, zoom_out, "zoom_out");
  GLADE_HOOKUP_OBJECT (main_window, orbit, "orbit");
  GLADE_HOOKUP_OBJECT (main_window, pan, "pan");
  GLADE_HOOKUP_OBJECT (main_window, center_selection, "center_selection");
  GLADE_HOOKUP_OBJECT (main_window, toolbar3, "toolbar3");
  GLADE_HOOKUP_OBJECT (main_window, do_sphere, "do_sphere");
  GLADE_HOOKUP_OBJECT (main_window, do_box, "do_box");
  GLADE_HOOKUP_OBJECT (main_window, do_cone, "do_cone");
  GLADE_HOOKUP_OBJECT (main_window, do_cylinder, "do_cylinder");
  GLADE_HOOKUP_OBJECT (main_window, hpaned1, "hpaned1");
  GLADE_HOOKUP_OBJECT (main_window, views, "views");
  GLADE_HOOKUP_OBJECT (main_window, vpaned1, "vpaned1");
  GLADE_HOOKUP_OBJECT (main_window, scrolll1, "scrolll1");
  GLADE_HOOKUP_OBJECT (main_window, scenegraph, "scenegraph");
  GLADE_HOOKUP_OBJECT (main_window, properties_scroll, "properties_scroll");
  GLADE_HOOKUP_OBJECT (main_window, properties_viewport, "properties_viewport");

  return main_window;
}  

GtkWidget*
create_warning_dialog (void)
{
  GtkWidget *warning_dialog;
  GtkWidget *dialog_vbox1;
  GtkWidget *hbox2;
  GtkWidget *image8;
  GtkWidget *message;
  GtkWidget *dialog_action_area1;
  GtkWidget *ok_warning_dialog;

  warning_dialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (warning_dialog), _("Warning!"));
  gtk_window_set_position (GTK_WINDOW (warning_dialog), GTK_WIN_POS_MOUSE);
  gtk_window_set_resizable (GTK_WINDOW (warning_dialog), FALSE);

  dialog_vbox1 = GTK_DIALOG (warning_dialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox2, TRUE, TRUE, 0);

  image8 = gtk_image_new_from_stock ("gtk-dialog-warning", GTK_ICON_SIZE_DIALOG);
  gtk_widget_show (image8);
  gtk_box_pack_start (GTK_BOX (hbox2), image8, FALSE, TRUE, 0);
  gtk_misc_set_padding (GTK_MISC (image8), 10, 10);

  message = gtk_label_new (_("Warning!"));
  gtk_widget_show (message);
  gtk_box_pack_start (GTK_BOX (hbox2), message, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (message), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (message), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (message), 10, 10);

  dialog_action_area1 = GTK_DIALOG (warning_dialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  ok_warning_dialog = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (ok_warning_dialog);
  gtk_dialog_add_action_widget (GTK_DIALOG (warning_dialog), ok_warning_dialog, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (ok_warning_dialog, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) warning_dialog, "delete_event",
                    G_CALLBACK (on_warning_dialog_delete_event),
                    NULL);
  g_signal_connect ((gpointer) ok_warning_dialog, "clicked",
                    G_CALLBACK (on_ok_warning_dialog_clicked),
                    NULL);

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (warning_dialog, warning_dialog, "warning_dialog");
  GLADE_HOOKUP_OBJECT_NO_REF (warning_dialog, dialog_vbox1, "dialog_vbox1");
  GLADE_HOOKUP_OBJECT (warning_dialog, hbox2, "hbox2");
  GLADE_HOOKUP_OBJECT (warning_dialog, image8, "image8");
  GLADE_HOOKUP_OBJECT (warning_dialog, message, "message");
  GLADE_HOOKUP_OBJECT_NO_REF (warning_dialog, dialog_action_area1, "dialog_action_area1");
  GLADE_HOOKUP_OBJECT (warning_dialog, ok_warning_dialog, "ok_warning_dialog");

  return warning_dialog;
}

GtkWidget*
create_preferences (void)
{
  GtkWidget *preferences;
  GtkWidget *vbox1;
  GtkWidget *label1;
  GtkWidget *config;

  preferences = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (preferences), _("Preferences"));

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (preferences), vbox1);

  label1 = gtk_label_new (_("The configuration won't take effect until you reload OSGedit,\nbecause these values define the defaults, and not the current\nsettings."));
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);
  gtk_misc_set_padding (GTK_MISC (label1), 10, 10);

  config = gtk_frame_new (NULL);
  gtk_widget_show (config);
  gtk_box_pack_start (GTK_BOX (vbox1), config, TRUE, TRUE, 0);
  gtk_frame_set_shadow_type (GTK_FRAME (config), GTK_SHADOW_NONE);

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (preferences, preferences, "preferences");
  GLADE_HOOKUP_OBJECT (preferences, vbox1, "vbox1");
  GLADE_HOOKUP_OBJECT (preferences, label1, "label1");
  GLADE_HOOKUP_OBJECT (preferences, config, "config");

  return preferences;
}
