#include <osg/GL>

#include <osg/Transform>
#include <osg/Billboard>
#include <osg/Geode>
#include <osg/Group>
#include <osg/Notify>

#include <osgDB/Registry>
#include <osgDB/ReadFile>

#include <osgGA/TrackballManipulator>
#include <osgGA/FlightManipulator>
#include <osgGA/DriveManipulator>

#include <osgUtil/Optimizer>

#include <osgProducer/Viewer>

#include "gtkviewer.h"
//#include "gtkrendersurface.h"

#ifndef WIN32
#  include <gdk/gdkx.h>
#endif

void write_usage(std::ostream& out,const std::string& name)
{
    out << std::endl;
    out <<"usage:"<< std::endl;
    out <<"    "<<name<<" [options] infile1 [infile2 ...]"<< std::endl;
    out << std::endl;
    out <<"options:"<< std::endl;
    out <<"    -l libraryName      - load plugin of name libraryName"<< std::endl;
    out <<"                          i.e. -l osgdb_pfb"<< std::endl;
    out <<"                          Useful for loading reader/writers which can load"<< std::endl;
    out <<"                          other file formats in addition to its extension."<< std::endl;
    out <<"    -e extensionName    - load reader/wrter plugin for file extension"<< std::endl;
    out <<"                          i.e. -e pfb"<< std::endl;
    out <<"                          Useful short hand for specifying full library name as"<< std::endl;
    out <<"                          done with -l above, as it automatically expands to"<< std::endl;
    out <<"                          the full library name appropriate for each platform."<< std::endl;
    out <<std::endl;
    out <<"    -stereo             - switch on stereo rendering, using the default of,"<< std::endl;
    out <<"                          ANAGLYPHIC or the value set in the OSG_STEREO_MODE "<< std::endl;
    out <<"                          environmental variable. See doc/stereo.html for "<< std::endl;
    out <<"                          further details on setting up accurate stereo "<< std::endl;
    out <<"                          for your system. "<< std::endl;
    out <<"    -stereo ANAGLYPHIC  - switch on anaglyphic(red/cyan) stereo rendering."<< std::endl;
    out <<"    -stereo QUAD_BUFFER - switch on quad buffered stereo rendering."<< std::endl;
    out <<std::endl;
    out <<"    -stencil            - use a visual with stencil buffer enabled, this "<< std::endl;
    out <<"                          also allows the depth complexity statistics mode"<< std::endl;
    out <<"                          to be used (press 'p' three times to cycle to it)."<< std::endl;
    out << std::endl;
}

/*void timeout(osgProducer::Viewer *viewer) {
    viewer->sync();
    viewer->update();
    viewer->frame();
}*/

void timeout(GtkViewer *viewer) {
    viewer->display();
}

int main(int argc, char *argv[]) {
    if (argc<2)
    {
        write_usage(std::cout,argv[0]);
        return 0;
    }
    // use an ArgumentParser object to manage the program arguments.
    osg::ArgumentParser arguments(&argc,argv);

    // set up the usage document, in case we need to print out how to use this program.
    arguments.getApplicationUsage()->setApplicationName(arguments.getApplicationName());
    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the GTK+OSG example which loads and visualises 3d models.");
    arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
    arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display this information");

    // if user request help write it out to cout.
    if (arguments.read("-h") || arguments.read("--help"))
    {
        arguments.getApplicationUsage()->write(std::cout);
        return 1;
    }

    // load the nodes from the commandline arguments.
    osg::Node* rootnode = osgDB::readNodeFiles(arguments);
    // if no model has been successfully loaded report failure.
    if (!rootnode)
    {
        std::cout << arguments.getApplicationName() <<": No data loaded" << std::endl;
        return 1;
    }
    // GTK_INIT jode completamente la funcion strtod haciendo que
    // trunque los numeros decimales, asi que luego no lee bien osg
    // Hay que leer antes del gtk_init
    gtk_init(&argc, &argv);
    
    // run optimization over the scene graph
    osgUtil::Optimizer optimzer;
    optimzer.optimize(rootnode);

    // Create a GTK Window and place the viewer into it
    GtkWidget *window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_usize(window,700,700);
    gtk_signal_connect(GTK_OBJECT(window),"delete_event",gtk_main_quit,0);
    gtk_widget_show(window);

    // initialize the viewer.
    GtkViewer viewer;

    // add a viewport to the viewer and attach the scene graph.
    viewer.addViewport( rootnode );
    
    // open the viewer window.
    viewer.open();

    gtk_container_add(GTK_CONTAINER(window), viewer.getWidget());

    gtk_timeout_add(33,(GtkFunction)(timeout),&viewer);
    // fire up the event loop.
    gtk_main();
    return 0;

    /*osgProducer::Viewer viewer;
    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
    viewer.setSceneData(rootnode);
    viewer.getCamera(0)->getRenderSurface()->setWindow( GDK_WINDOW_XWINDOW(window->window) );
    //GtkRenderSurface *surf=new GtkRenderSurface();
    //gtk_container_add(GTK_CONTAINER(window),surf->getWidget());
    //viewer.getCamera(0)->setRenderSurface(surf);
    viewer.realize();

    gtk_timeout_add(33,(GtkFunction)(timeout),&viewer);
    gtk_main();*/
}
